@@ -1,6 +1,7 @@
 import copy
 import os
 import torch
+import numpy as np
 from tqdm.autonotebook import tqdm
 from torch.optim.lr_scheduler import _LRScheduler
 import matplotlib.pyplot as plt
@@ -30,9 +31,9 @@ class DataLoaderIter(object):
     def inputs_labels_from_batch(self, batch_data):
         if not isinstance(batch_data, list) and not isinstance(batch_data, tuple):
             raise ValueError(
-                "Your batch type not supported: {}. Please inherit from "
-                "`TrainDataLoaderIter` (or `ValDataLoaderIter`) and redefine "
-                "`_batch_make_inputs_labels` method.".format(type(batch_data))
+                "Your batch type is not supported: {}. Please inherit from "
+                "`TrainDataLoaderIter` or `ValDataLoaderIter` and override the "
+                "`inputs_labels_from_batch` method.".format(type(batch_data))
             )
 
         inputs, labels, *_ = batch_data
@@ -67,7 +68,40 @@ class TrainDataLoaderIter(DataLoaderIter):
 
 
 class ValDataLoaderIter(DataLoaderIter):
-    pass
+    """This iterator will reset itself **only** when it is acquired by
+    the syntax of normal `iterator`. That is, this iterator just works
+    like a `torch.data.DataLoader`. If you want to restart it, you
+    should use it like:
+
+        ```
+        loader_iter = ValDataLoaderIter(data_loader)
+        for batch in loader_iter:
+            ...
+
+        # `loader_iter` should run out of values now, you can restart it by:
+        # 1. the way we use a `torch.data.DataLoader`
+        for batch in loader_iter:        # __iter__ is called implicitly
+            ...
+
+        # 2. passing it into `iter()` manually
+        loader_iter = iter(loader_iter)  # __iter__ is called by `iter()`
+        ```
+    """
+
+    def __init__(self, data_loader):
+        super().__init__(data_loader)
+        self.run_limit = len(self.data_loader)
+        self.run_counter = 0
+
+    def __iter__(self):
+        if self.run_counter >= self.run_limit:
+            self._iterator = iter(self.data_loader)
+            self.run_counter = 0
+        return self
+
+    def __next__(self):
+        self.run_counter += 1
+        return super(ValDataLoaderIter, self).__next__()
 
 
 class LRFinder(object):
@@ -396,19 +430,22 @@ class LRFinder(object):
                     inputs, labels, non_blocking=non_blocking_transfer
                 )
 
-                if isinstance(inputs, tuple) or isinstance(inputs, list):
-                    batch_size = inputs[0].size(0)
-                else:
-                    batch_size = inputs.size(0)
-
                 # Forward pass and loss computation
                 outputs = self.model(inputs)
                 loss = self.criterion(outputs, labels)
-                running_loss += loss.item() * batch_size
+                running_loss += loss.item() * len(labels)
 
         return running_loss / len(val_iter.dataset)
 
-    def plot(self, skip_start=10, skip_end=5, log_lr=True, show_lr=None, ax=None):
+    def plot(
+        self,
+        skip_start=10,
+        skip_end=5,
+        log_lr=True,
+        show_lr=None,
+        ax=None,
+        suggest_lr=True,
+    ):
         """Plots the learning rate range test.
 
         Arguments:
@@ -424,9 +461,13 @@ class LRFinder(object):
                 matplotlib axes object and the figure is not be shown. If `None`, then
                 the figure and axes object are created in this method and the figure is
                 shown . Default: None.
+            suggest_lr (bool, optional): suggest a learning rate by
+                - 'steepest': the point with steepest gradient (minimal gradient)
+                you can use that point as a first guess for an LR. Default: True.
 
         Returns:
-            The matplotlib.axes.Axes object that contains the plot.
+            The matplotlib.axes.Axes object that contains the plot,
+            and the suggested learning rate (if set suggest_lr=True).
         """
 
         if skip_start < 0:
@@ -454,6 +495,31 @@ class LRFinder(object):
 
         # Plot loss as a function of the learning rate
         ax.plot(lrs, losses)
+
+        # Plot the suggested LR
+        if suggest_lr:
+            # 'steepest': the point with steepest gradient (minimal gradient)
+            print("LR suggestion: steepest gradient")
+            min_grad_idx = None
+            try:
+                min_grad_idx = (np.gradient(np.array(losses))).argmin()
+            except ValueError:
+                print(
+                    "Failed to compute the gradients, there might not be enough points."
+                )
+            if min_grad_idx is not None:
+                print("Suggested LR: {:.2E}".format(lrs[min_grad_idx]))
+                ax.scatter(
+                    lrs[min_grad_idx],
+                    losses[min_grad_idx],
+                    s=75,
+                    marker="o",
+                    color="red",
+                    zorder=3,
+                    label="steepest gradient",
+                )
+                ax.legend()
+
         if log_lr:
             ax.set_xscale("log")
         ax.set_xlabel("Learning rate")
@@ -466,7 +532,10 @@ class LRFinder(object):
         if fig is not None:
             plt.show()
 
-        return ax
+        if suggest_lr and min_grad_idx:
+            return ax, lrs[min_grad_idx]
+        else:
+            return ax
 
 
 class LinearLR(_LRScheduler):
